#include <stdio.h>
#include <stdlib.h>

#include <ao/ao.h>

/******************************************************************************/

void err_exit( char * msg )
    {
    fprintf(stderr, "FATAL ERROR: %s\n", msg ? msg : "Unknown error" );
    exit( EXIT_FAILURE );
    }


/******************************************************************************/

int main ( int argc, char** argv ) 
    {
    ao_initialize();

    ao_info ** driver_list = NULL;
    int driver_count = -1;

    driver_list = ao_driver_info_list( &driver_count );
    if ( driver_list == NULL )
        {
        err_exit("Unable to get list of drivers");
        }

    printf( "Endianness: %s\n", ao_is_big_endian() ? "big" : "little" );

    int default_driver_id = ao_default_driver_id();
    if ( -1 == default_driver_id )
        {
        printf( "No default driver available\n" );
        }

    for ( int i = 0; i < driver_count; ++i )
        {
        printf( "%s\n", driver_list[i]->name );
        int driver_id = ao_driver_id( driver_list[i]->short_name );
        if ( default_driver_id == driver_id )
            {
            printf( "\tDEFAULT DRIVER\n" );
            }
        printf( "\tshort name           :\t%s\n", driver_list[i]->short_name );
        printf( "\ttype                 :\t%s\n",
            driver_list[i]->type == AO_TYPE_LIVE ? "live" :
            (driver_list[i]->type == AO_TYPE_FILE ? "file" :
            "unknown")
            );
        printf( "\tpriority             :\t%d\n", driver_list[i]->priority );
        printf( "\tpreferred byte format:\t%s\n",
            driver_list[i]->preferred_byte_format == AO_FMT_LITTLE ? "little" : 
            (driver_list[i]->preferred_byte_format == AO_FMT_BIG ? "big" : "unknown" )
            );

#if 0
        // Appears that ao-file_extension is not in my version of libao
        char * ext = NULL;
        ext = ao_file_extension( driver_id );
        printf( "\tnormal file extension:\t%s\n", ext == NULL ? "[none]" : ext );
#endif

        printf( "\toptions              :\t", driver_list[i]->comment);
        for ( int j = 0; j < driver_list[i]->option_count ; ++j )
            {
            printf( "%s ", driver_list[i]->options[j]);
            }
        printf("\n");

        printf( "\tcomment              :\t%s\n", driver_list[i]->comment);
        }

    ao_shutdown();
    }

